# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Weak abstractions of connections of a master to nodes.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import json
import socket

import gnutls.crypto
from gnutls.crypto import *
from gnutls.connection import *

from mapwoc.lib.mapwoc_json import mapwoc_from_json_hook
from mapwoc.lib.node import Node
from mapwoc.lib.trivialresponse import TrivialResponse

# TODO use logging

class NodeConnection(Node):
    """Object to query a Node."""

    def __init__ (self, filename, credentials=None):
        """Read information about how to connecto to a node from a keyfile."""
        Node.__init__(self, filename) 
        self.credentials = credentials

    def ask(self, question):
        """
        Connect to node, send a string and receive answer.
        @param question: Question to be asked.
        @return: Answer received.
        """

        try:
            # Create a socket (SOCK_STREAM means a TCP socket).
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

            if self.credentials:
                session = ClientSession(sock, self.credentials)
                session.connect((self.ip, int(self.port)))
                session.handshake()
                session.send(question + "\r\n")
                # TODO receive-all mess.
                received = session.recv(16384)
                #logger.debug("Peer certificate: %s" % str(session.peer_certificate))
                print "Peer certificate: %s" % str(session.peer_certificate)
                session.bye()
                session.close()
            else:
                # Connect to server and send the question.
                sock.connect((self.ip, int(self.port)))
                sock.send(question + "\r\n")

                # TODO lessen assumption about amount of data transmitted (twisted?)
                # Receive data from the server and shut down.
                sock.settimeout(30)
                received = sock.recv(16384, socket.MSG_WAITALL)

                sock.close()
        except socket.error as err:
            print "Exception: while talking to node %s:%s Error: %s" % \
                    (self.ip, self.port, str(err))
            print "Question was: %s " % question
            received = "Connection Error: " + str(err)

        return received

    def ask_decode(self, question):
        """
        Asks question to node, returns deserialized answer.
        """
        answer = self.ask(question)
        try:
            dec = json.loads(answer, object_hook=mapwoc_from_json_hook)
            return dec
        except Exception as e:
            print("panic: communication with node failed: (%s) %s "\
                    % (str(answer),str(e)))
            # TODO handle error
           # raise

    def ask_ok_tr(self, question):
        """
        Asks node for a trivial response and returns answer.
        @param question: Question to ask.
        @return: A pair, where the first element is True if no error is
                 indicated by the TrivialResponse, which is the second
                 element in the pair.
        """
        tr = self.ask_decode(question)
        ok = isinstance(tr, TrivialResponse) and not tr.error 
        return ok,tr
